@@ -9,8 +9,12 @@ lca <-  function(data,series=names(data$rate)[1],years=data$year, ages=data$age,
     chooseperiod=FALSE, minperiod=20, breakmethod=c("bai","bms"),
     scale=FALSE, restype=c("logrates","rates","deaths"), interpolate=FALSE)
 {
-    if(class(data) != "demogdata" | data$type != "mortality")
-        stop("Not mortality data")
+  if (class(data) != "demogdata") {
+    stop("Not demography data")
+  }
+  if (!any(data$type == c("mortality", "fertility"))) {
+    stop("Neither mortality nor fertility data")
+  }
 
     adjust <- match.arg(adjust)
     restype <- match.arg(restype)
@@ -35,10 +39,12 @@ lca <-  function(data,series=names(data$rate)[1],years=data$year, ages=data$age,
         stopyear <- min(stopyear,max(data$year))
     else
         stopyear <- max(data$year)
-    id2 <- match(startyear:stopyear,data$year)
+    id2 <- na.omit(match(startyear:stopyear,data$year))
+
     mx <- mx[,id2]
     pop <- pop[,id2]
     year <- data$year[id2]
+    deltat <- year[2]-year[1]
     ages <- data$age
     n <- length(ages)
     m <- sum(id2>0)
@@ -68,7 +74,7 @@ lca <-  function(data,series=names(data$rate)[1],years=data$year, ages=data$age,
     # Do SVD
     ax <- apply(logrates,2,mean) # ax is mean of logrates by column
     if(sum(ax < -1e9) > 0)
-        stop("Some mortality rates are zero.\n Try reducing the maximum age or setting interpolate=TRUE.")
+        stop(sprintf("Some %s rates are zero.\n Try reducing the maximum age or setting interpolate=TRUE.", data$type))
     clogrates <- sweep(logrates,2,ax) # central log rates (with ax subtracted) (dimensions m*n)
     svd.mx <- svd(clogrates)
 
@@ -162,7 +168,7 @@ lca <-  function(data,series=names(data$rate)[1],years=data$year, ages=data$age,
             bestbreak <- order(RS[1:(m-minperiod)])[1]-1
             out <- lca(data,series,year[(bestbreak+1):m],ages=ages,max.age=max.age,
                 adjust=adjust,chooseperiod=FALSE,interpolate=interpolate,scale=scale)
-            out$mdevs <- ts(cbind(devlin,devadd,RS),start=startyear,frequency=1)
+            out$mdevs <- ts(cbind(devlin,devadd,RS),start=startyear,deltat=deltat)
             dimnames(out$mdevs)[[2]] <- c("Mean deviance total","Mean deviance base","Mean deviance ratio")
             return(out)
         }
@@ -185,8 +191,10 @@ lca <-  function(data,series=names(data$rate)[1],years=data$year, ages=data$age,
         fit <- exp(logfit)*pop
         res <- deaths - fit
     }
-    residuals <- fts(ages,t(res),frequency=1,start=years[1],xname="Age",yname="Residuals mortality rate")
-    fitted <- fts(ages,t(fit),frequency=1,start=years[1],xname="Age",yname="Fitted mortality rate")
+    residuals <- fts(ages,t(res),frequency=1/deltat,start=years[1],xname="Age",
+                     yname=paste("Residuals", data$type, "rate"))
+    fitted <- fts(ages,t(fit),frequency=1/deltat,start=years[1],xname="Age",
+                  yname=paste("Fitted", data$type, "rate"))
 
     names(ax) <- names(bx) <- ages
 
@@ -211,13 +219,16 @@ lca <-  function(data,series=names(data$rate)[1],years=data$year, ages=data$age,
 
     #Return
     output <- list(label=data$label,age=ages,year=year, mx=t(mx),
-        ax=ax, bx=bx, kt=ts(kt,start=startyear,frequency=1), residuals=residuals, fitted=fitted,
-        varprop=svd.mx$d[1]^2/sum(svd.mx$d^2), y=fts(ages,t(mx),start=years[1],frequency=1,xname="Age",yname="Mortality"),
+        ax=ax, bx=bx, kt=ts(kt,start=startyear,deltat=deltat), residuals=residuals, fitted=fitted,
+        varprop=svd.mx$d[1]^2/sum(svd.mx$d^2), 
+        y=fts(ages,t(mx),start=years[1],frequency=1/deltat,xname="Age",
+              yname=ifelse(data$type == "mortality", "Mortality", "Fertility")),
         mdev=mdev)
     names(output)[4] <- series
     output$call <- match.call()
     names(output$mdev) <- c("Mean deviance base","Mean deviance total")
     output$adjust <- adjust
+    output$type <- data$type
     return(structure(output,class="lca"))
 }
 
@@ -282,10 +293,10 @@ summary.lca <- function(object,...)
 {
     print(object)
 
-    cat("\nERROR MEASURES BASED ON MORTALITY RATES\n")
+    cat(sprintf("\nERROR MEASURES BASED ON %s RATES\n", toupper(object$type)))
     printout(fdmMISE(object[[4]],exp(object$fitted$y),age=object$y$x,years=object$year))
 
-    cat("\nERROR MEASURES BASED ON LOG MORTALITY RATES\n")
+    cat(sprintf("\nERROR MEASURES BASED ON LOG %s RATES\n", toupper(object$type)))
     printout(fdmMISE(log(object[[4]]),object$fitted$y,age=object$y$x,years=object$year))
 }
 
@@ -345,7 +356,8 @@ forecast.lca <- function(object, h=50, se=c("innovdrift","innovonly"), jumpchoic
     kt.hi.forecast <- kt.forecast + (zval*kt.stderr)
     kt.f <- data.frame(kt.forecast,kt.lo.forecast,kt.hi.forecast)
     names(kt.f) <- c("kt forecast","kt lower","kt upper")
-    kt.f <- ts(kt.f,start=object$year[nyears]+1)
+    deltat <- object$year[2] - object$year[1]
+    kt.f <- ts(kt.f,start=object$year[nyears]+deltat,deltat=deltat)
 
     # Calculate expected life and mx forecasts
     e0.forecast <- rep(0,h)
@@ -362,15 +374,15 @@ forecast.lca <- function(object, h=50, se=c("innovdrift","innovonly"), jumpchoic
     }
     kt.f <- data.frame(kt.forecast,kt.lo.forecast,kt.hi.forecast)
     names(kt.f) <- c("kt forecast","kt lower","kt upper")
-    kt.f <- ts(kt.f,start=object$year[nyears]+1)
+    kt.f <- ts(kt.f,start=object$year[nyears]+deltat,deltat=deltat)
 
-    output = list(label=object$label,age=object$age,year=object$year[nyears]+x,
+    output = list(label=object$label,age=object$age,year=object$year[nyears] + x*deltat,
             rate=list(forecast=mx.forecast,lower=mx.lo.forecast,upper=mx.hi.forecast),
             fitted=object$fitted,
-            e0=ts(e0.forecast,frequency=1,start=object$year[nyears]+1),
+            e0=ts(e0.forecast,start=object$year[nyears]+deltat,deltat=deltat),
             kt.f=structure(list(mean=kt.f[,1],lower=kt.f[,2],upper=kt.f[,3],level=level,x=object$kt,
                                 method="Random walk with drift"),class="forecast"),
-                type = "mortality",lambda=0)
+                type = object$type,lambda=0)
     names(output$rate)[1] = names(object)[4]
     output$model <- object
     output$model$jumpchoice <- jumpchoice
